// Simulates hitting the drum heads
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module DrumHitter (
	iCLK,
	iNewFrame,
	iFrameCount,
	iBufferFrameCount,
	iNotes,
	iPedals,
	iValid,
	oNotes,
	oPedals);
	
	input 		iCLK;
	input			iNewFrame;				// Signals start of new frame
	input	[7:0]	iFrameCount;			// Global frame count
	input [7:0]	iBufferFrameCount;	// Frame to hit next note on
	input	[3:0]	iNotes, iPedals;		// Notes to hit (from buffer)

	input			iValid;	// Is buffer output valid?
	
	output reg [3:0]	oNotes, oPedals;// Output to drums
	
	reg	[2:0]	EndHitCounter;		// Counter to end strike signal
	
	// Hits heads at buffer frame, end 3 frames after
	always @(posedge iCLK) begin
		oNotes <= ((iFrameCount==(iBufferFrameCount-8'd0))&&iValid&&iNewFrame) ? iNotes : (
					 ((EndHitCounter==3'd0)&&iNewFrame)        ? 4'b0   : oNotes);
		oPedals <= ((iFrameCount==(iBufferFrameCount-8'd0))&&iValid&&iNewFrame) ? iPedals : (
					 ((EndHitCounter==3'd0)&&iNewFrame)        ? 4'b0   : oPedals);
					 
		EndHitCounter <= (((iFrameCount==(iBufferFrameCount-8'd0))&&iValid&&iNewFrame) ? 3'd2 : (
								 (iNewFrame) ? EndHitCounter-3'd1 : EndHitCounter));
	end
	
endmodule
