//Guitar module outputs note and strum signals
module Drums (
	iCLK_27,
	iReset,
	iNewFrame,
	iFrameCount,
	iRed, iGreen, iBlue,
	iVGA_X, iVGA_Y,
	iVGA_BLANK,
	iGxy,iRxy,iYxy,iBxy,iOxy,
	oNotes,
	oPedals,
	pedal_LED,
	oStrum );
	
	input				iCLK_27;								//27MHz clock
	input				iReset;
	input				iNewFrame;
	input	[7:0]		iFrameCount;
	input	[9:0]		iRed, iGreen, iBlue;				//RGB signal from decoder
	input	[9:0]		iVGA_X, iVGA_Y;					//Current pixel location
	input				iVGA_BLANK;							//VGA blank (active low)
	input	[19:0]	iGxy, iRxy, iYxy, iBxy, iOxy;	//Input X and Y coordinates for Detection location for
																//the 5 colors
	output [3:0]	oNotes;								//Note outputs
	output [3:0]	oPedals;
	output 			oStrum;								//Pedal signal
		
	wire 	[3:0]	notes_bd0;			//Note signals given to buffer from bar detector
	wire  [3:0] pedals_bd0;			//Pedal signals given to buffer from bad detector
	output wire  [3:0] pedal_LED;	//Pedal LED output for debugging
	wire	[3:0]	notes_buff0;		//Note signals given to pedalplayer from buffer
	wire	[3:0]	pedals_buff0;		//Pedal signals given to pedalplayer from buffer
	wire	[7:0]	frameNum_bd0;		//Frame number given to buffer from bar detector
	wire	[7:0]	frameNum_buff0;	//Frame number from buffer to pedalplayer/drumhitter
	wire			writeBuffer_bd0;	//Signal write to buffer from Bar detector
	wire			readBuffer_pedal;	//Signal read to buffer from pedalplayer
	wire			valid_buff0;		//Current output is valid (buffer is not empty)
	
	//Bar detector detects new notes to strum
	Bar_Detector_Drum bdd0 (
		.iCLK_27			(iCLK_27),
		.iNewFrame		(iNewFrame),
		.iFrameCount	(iFrameCount),
		.iRed				(iRed), 
		.iGreen			(iGreen), 
		.iBlue			(iBlue),
		.iVGA_X			(iVGA_X),
		.iVGA_Y			(iVGA_Y),
		.iVGA_BLANK		(iVGA_BLANK),
		.iGxy				(iGxy),
		.iRxy				(iRxy),
		.iYxy				(iYxy),
		.iBxy				(iBxy),
		.oNotes			(notes_bd0),
		.oPedals			(pedals_bd0),
		.oPedalLED		(pedal_LED),
		.oFrameNum		(frameNum_bd0),
		.oWriteBuffer	(writeBuffer_bd0)
	);
	
	//Buffer stores notes to be checked at next detector
	Ring_Buffer_Drums rbd0 (
		.iCLK_27			(iCLK_27),
		.iReset			(iReset),
		.iNotes			(notes_bd0),
		.iPedals			(pedals_bd0),
		.iFrameNum		(frameNum_bd0),
		.iWriteBuffer	(writeBuffer_bd0),
		.iReadBuffer	(readBuffer_pedal),
		.oNotes			(notes_buff0),
		.oPedals			(pedals_buff0),
		.oFrameNum		(frameNum_buff0),
		.oValid			(valid_buff0)
	);
	
	//Strum module prevents false triggering
	PedalPlayer pp0 (
		.iCLK					(iCLK_27),
		.iNewFrame			(iNewFrame),
		.iFrameCount		(iFrameCount),
		.iBufferFrameCount(frameNum_buff0),
		.iValid				(valid_buff0),
		.iNotes				(notes_buff0),
		.iPedals				(pedals_buff0),
		.oRead				(readBuffer_pedal),
		.oStrum				(oStrum)
	);
	
	//DrumHitter module hits buttons
	DrumHitter dh0 (
		.iCLK						(iCLK_27),
		.iNewFrame				(iNewFrame),
		.iFrameCount			(iFrameCount),
		.iBufferFrameCount	(frameNum_buff0),
		.iNotes					(notes_buff0),
		.iPedals					(pedals_buff0),
		.iValid					(valid_buff0),
		.oNotes					(oNotes),
		.oPedals					(oPedals)
	);
	
endmodule
	