// Signals start of a new frame and keeps frame counter
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module FrameCounter (
	iCLK,
	iReset,
	iVGA_X, iVGA_Y,
	iVGA_BLANK,
	oNewFrame,
	oFrameCount );
	
	input			iCLK;
	input			iReset;
	input	[9:0]	iVGA_X, iVGA_Y;	// Current pixel location
	input			iVGA_BLANK;			// High when VGA is reading pixels
	
	output				oNewFrame;	// Signals start of new frame for 1 cycle
	output reg	[7:0]	oFrameCount;// Arbitrary frame counter
	
	// Check VGA_BLANK to make sure it is actually starting a new frame
	assign oNewFrame = (iVGA_X==10'd0 && iVGA_Y==10'd0 && iVGA_BLANK);
	
	always @(posedge iCLK) begin
		if (iReset) begin
			oFrameCount <= 8'd0;
		end
		else begin
			oFrameCount <= oNewFrame ? (oFrameCount + 8'd1) : oFrameCount;
		end
	end
	
endmodule
