// A grid of pixel detectors to check for a color in a square area
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module Grid_Detector_Drum (
	iCLK_27,
	iRed, iGreen, iBlue,
	iVGA_X, iVGA_Y,
	iVGA_BLANK,
	iBotLeft,
	iColorSel,
	oColor,
	oPedal);
	
	input				iCLK_27;					//	27MHz clock
	input	[9:0]		iRed, iGreen, iBlue;	//	RGB signal from decoder
	input	[9:0]		iVGA_X, iVGA_Y;		//	Current pixel location
	input				iVGA_BLANK;				//	High when VGA is reading pixels
	input	[19:0]	iBotLeft;				// Bottom-Left of pixel grid box {x,y}
	input [2:0]		iColorSel;				// Selects which color to detect
	
	output wire	oColor;
	output wire oPedal;
	
	`define WIDTH 3	// Width of grid in pixels
	`define HEIGHT 3	// Height of grid in pixels
	
	parameter GREEN = 3'd0, RED = 3'd1, YELLOW = 3'd2, BLUE = 3'd3, ORANGE = 3'd4;

	wire	[9:0]	BL_X, BL_Y;
	wire [`WIDTH*`HEIGHT-1:0] Pixels;
	wire [`WIDTH*`HEIGHT-1:0] Pedals;
	
	assign BL_X = iBotLeft[19:10];
	assign BL_Y = iBotLeft[9:0];
	
	assign oColor = &Pixels;	// All pixels must detect for the grid to flag a detect
	assign oPedal = &Pedals;	// All pixels must detect bass pedal or note for the flag to trigger

	//Generate pixel detector grid
	genvar x;
	genvar y;
	generate
			for(x = 0; x < `WIDTH; x = x + 1) begin:xFor
				for(y = 0; y < `HEIGHT; y = y + 1) begin:yFor
					Pixel_Detector_Drum pd(
						.iCLK_27		(iCLK_27),
						.iRed			(iRed),
						.iGreen		(iGreen),
						.iBlue		(iBlue),
						.iVGA_X		(iVGA_X),
						.iVGA_Y		(iVGA_Y),
						.iVGA_BLANK	(iVGA_BLANK),
						.iX			({22'b0,BL_X}+x),
						.iY			({22'b0,BL_Y}-y),
						.iColorSel	(iColorSel),
						.oPixel		(Pixels[y*`WIDTH + x]),
						.oPedal		(Pedals[y*`WIDTH + x])
					);
				end
			end
	endgenerate
endmodule
