//Guitar module outputs note and strum signals
module Guitar (
	iCLK_27,
	iReset,
	iNewFrame,
	iFrameCount,
	iRed, iGreen, iBlue,
	iVGA_X, iVGA_Y,
	iVGA_BLANK,
	iGxy,iRxy,iYxy,iBxy,iOxy,
	oNotes,
	oStrum );
	
	input				iCLK_27;								//27MHz clock
	input				iReset;
	input				iNewFrame;
	input	[7:0]		iFrameCount;
	input	[9:0]		iRed, iGreen, iBlue;				//RGB signal from decoder
	input	[9:0]		iVGA_X, iVGA_Y;					//Current pixel location
	input				iVGA_BLANK;							//VGA blank (active low)
	input	[19:0]	iGxy, iRxy, iYxy, iBxy, iOxy;	//Input X and Y coordinates for Detection location for						
																//the 5 colors	
	output [4:0]	oNotes;		//Note outputs
	output 			oStrum;					//Strum signal
		
	wire 	[4:0]	notes_bd0;			//Note signals given to buffer from bar detector
	wire	[4:0]	notes_buff0;		//Note signals given to strummer from buffer
	wire	[7:0]	frameNum_bd0;		//Frame number given to buffer from bar detector
	wire	[7:0]	frameNum_buff0;	//Frame number from buffer
	wire			writeBuffer_bd0;	//Signal write to buffer from Bar detector
	wire			readBuffer_strum;	//Signal read from buffer to strummer
	wire			valid_buff0;		//Current output is valid (buffer is not empty)
	
	wire 	[4:0]	notes_bd1;			//Note signals given to buffer from bar detector
	wire	[4:0]	notes_buff1;		//Note signals given to strummer from buffer
	wire	[7:0]	frameNum_bd1;		//Frame number given to buffer from bar detector
	wire	[7:0]	frameNum_buff1;	//Frame number from buffer
	wire			writeBuffer_bd1;	//Signal write to buffer from Bar detector
	wire			valid_buff1;		//Current output is valid (buffer is not empty)
	
	//Bar detector detects new notes to strum
	Bar_Detector bd0 (
		.iCLK_27			(iCLK_27),
		.iNewFrame		(iNewFrame),
		.iFrameCount	(iFrameCount),
		.iRed				(iRed), 
		.iGreen			(iGreen), 
		.iBlue			(iBlue),
		.iVGA_X			(iVGA_X),
		.iVGA_Y			(iVGA_Y),
		.iVGA_BLANK		(iVGA_BLANK),
		.iGxy				(iGxy),
		.iRxy				(iRxy),
		.iYxy				(iYxy),
		.iBxy				(iBxy),
		.iOxy				(iOxy),
		.oNotes			(notes_bd0),
		.oFrameNum		(frameNum_bd0),
		.oWriteBuffer	(writeBuffer_bd0)
	);
	
	//Buffer stores notes to be checked at next detector
	Ring_Buffer rb0 (
		.iCLK_27			(iCLK_27),
		.iReset			(iReset),
		.iNotes			(notes_bd0),
		.iFrameNum		(frameNum_bd0),
		.iWriteBuffer	(writeBuffer_bd0),
		.iReadBuffer	(readBuffer_strum),
		.oNotes			(notes_buff0),
		.oFrameNum		(frameNum_buff0),
		.oValid			(valid_buff0)
	);
	
	//Strum module prevents false triggering
	Strummer s0 (
		.iCLK					(iCLK_27),
		.iNewFrame			(iNewFrame),
		.iFrameCount		(iFrameCount),
		.iBufferFrameCount(frameNum_buff0),
		.iValid				(valid_buff0),
		.oRead				(readBuffer_strum),
		.oStrum				(oStrum)
	);
	
	//FretPresser module holds buttons down
	FretPresser fp0 (
		.iCLK						(iCLK_27),
		.iNewFrame				(iNewFrame),
		.iFrameCount			(iFrameCount),
		.iBufferFrameCount	(frameNum_buff0),
		.iNotes					(notes_buff0),
		.iValid					(valid_buff0),
		.oNotes					(oNotes)
	);
	
endmodule
	