module Note_LUT (
	iCLK_27,
	iPx_v, iPx_n,
	iDir,
	iNewFrame,
	iLockIn,
	oIncr, 
	oLevel);

	input				iCLK_27;
	input	[31:0]	iPx_v, iPx_n;
	input				iDir;
	input				iNewFrame;
	input				iLockIn;
	
	output reg	[31:0]	oIncr;
	output	[5:0]	oLevel;
	assign oLevel = Lvl_adjust;
	
	reg	[5:0]	Level;
	reg	[3:0]	frameCount;
	reg	[3:0]	Lvl_adjust;
	
	wire				belowThresh;	// Not enough pixels difference between voice and note
	wire				areZero;			// One of the detection arrays is zero
	wire				onePixel;		// Voice is 1 pixel low from note
	wire	[31:0]	pixel_array;	
	
	assign areZero = iPx_v==32'd0 || iPx_n==32'd0;
	assign onePixel = (iPx_v[0] && iPx_n[1] && ~iPx_n[0]);
	
	assign pixel_array = iDir ? iPx_n : (iPx_v>>4);
	
	
	always @(pixel_array) begin
		casex (pixel_array)
			32'bxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx1: Lvl_adjust = 6'd0;
			32'bxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx10: Lvl_adjust = 6'd1;
			32'bxxxxxxxxxxxxxxxxxxxxxxxxxxxxx100: Lvl_adjust = 6'd1;
			32'bxxxxxxxxxxxxxxxxxxxxxxxxxxxx1000: Lvl_adjust = 6'd1;
			32'bxxxxxxxxxxxxxxxxxxxxxxxxxxx10000: Lvl_adjust = 6'd2;
			32'bxxxxxxxxxxxxxxxxxxxxxxxxxx100000: Lvl_adjust = 6'd2;
			32'bxxxxxxxxxxxxxxxxxxxxxxxxx1000000: Lvl_adjust = 6'd2;
			32'bxxxxxxxxxxxxxxxxxxxxxxxx10000000: Lvl_adjust = 6'd3;
			32'bxxxxxxxxxxxxxxxxxxxxxxx100000000: Lvl_adjust = 6'd3;
			32'bxxxxxxxxxxxxxxxxxxxxxx1000000000: Lvl_adjust = 6'd3;
			32'bxxxxxxxxxxxxxxxxxxxxx10000000000: Lvl_adjust = 6'd3;
			32'bxxxxxxxxxxxxxxxxxxxx100000000000: Lvl_adjust = 6'd3;
			32'bxxxxxxxxxxxxxxxxxxx1000000000000: Lvl_adjust = 6'd3;
			32'bxxxxxxxxxxxxxxxxxx10000000000000: Lvl_adjust = 6'd3;
			default:								  	 	  Lvl_adjust = 6'd3;
		endcase
	end
	
	always @(Level) begin
		case (Level)
			6'd0:		oIncr = 32'd18898;	//A3 	220.00Hz
			6'd1:		oIncr = 32'd20021;	//A3#	233.08Hz
			6'd2:		oIncr = 32'd21212;	//B3	246.94Hz
			6'd3:		oIncr = 32'd22474;	//C4	261.63Hz
			6'd4:		oIncr = 32'd23810;	//C4# 277.18Hz
			6'd5:		oIncr = 32'd25255;	//D4	293.66Hz
			6'd6:		oIncr = 32'd26726;	//D4#	311.13Hz
			6'd7:		oIncr = 32'd28315;	//E4	329.63Hz
			6'd8:		oIncr = 32'd29999;	//F4	349.23Hz
			6'd9:		oIncr = 32'd31782;	//F4#	369.99Hz
			6'd10:	oIncr = 32'd33672;	//G4	392.00Hz
			6'd11:	oIncr = 32'd35674;	//G4#	415.30Hz
			default:	oIncr = 32'd0;			//NO HERTZ
		endcase
	end
	
	wire [5:0] Sum, Diff;
	assign Sum  = Level+Lvl_adjust;
	assign Diff = Level-Lvl_adjust;
	
	// Updates the level
	always @(posedge iCLK_27) begin
		if (iNewFrame & ~iLockIn & ~areZero && frameCount==4'd0) begin
			if (iDir) begin
				Level <= (Sum>6'd11) ? (Sum-6'd12) : (Sum);
			end
			else begin
				Level <= (Diff[5]) ? (Diff+6'd12) : Diff; //If Diff overflows negative, add to get back in range
			end
			frameCount <= 4'd10;
		end
		else if (iNewFrame) begin
			frameCount <= (frameCount==4'd0) ? (4'd0) : (frameCount-4'd1);
		end
		else begin
			Level <= Level;
		end
	end
endmodule
