// Strums on frame match from buffer output
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module PedalPlayer (
	iCLK,
	iNewFrame,
	iFrameCount,
	iBufferFrameCount,
	iValid,
	iNotes,
	iPedals,
	oRead,
	oStrum );
	
	input 		iCLK;
	input			iNewFrame;				// Signals a new frame
	input	[7:0]	iFrameCount;			//	Global frame counter
	input	[7:0]	iBufferFrameCount;	//	Frame value of next note to hit
	input			iValid;					// Is the head value of buffer valid?
	input [3:0] iNotes, iPedals;		// Notes and Pedals (to determine if bass pedal should be hit)

	output reg oRead;		// Tells buffer to increment head after a read
	output reg oStrum;	// Pedal output to drums
	
	reg	[2:0] Hold;	// History used to hold pedal for 3 frames
	
	wire	isPedal;
	assign isPedal = (iPedals[3])&(iPedals[2])&(iPedals[1])&(iPedals[0]);
	
	// On a frame count match with buffer, hold pedal high for 3 frames
	always @(posedge iCLK) begin
		if (iNewFrame && Hold==3'd0) begin
			oStrum <= ((iFrameCount==(iBufferFrameCount))&&iValid&&isPedal) ? 1'b1 : 1'b0;
			oRead  <= ((iFrameCount==(iBufferFrameCount))&&iValid) ? 1'b1 : 1'b0;
			Hold	 <= ((iFrameCount==(iBufferFrameCount))&&iValid&&isPedal) ? 3'd2 : 3'd0;
		end
		else if (iNewFrame && Hold>3'd0) begin
			oStrum <= 1'b1;
			oRead  <= 1'b0;
			Hold	 <= Hold-3'd1;
		end
		else begin
			oStrum <= oStrum;
			oRead  <= 1'b0;
		end
	end
	
endmodule
