// Detects when a pixel turns a specified color
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module Pixel_Detector (
	iCLK_27,
	iRed, iGreen, iBlue,
	iVGA_X, iVGA_Y,
	iVGA_BLANK,
	iX, iY,
	iColorSel,
	oPixel);
	
	input				iCLK_27;					// 27MHz clock
	input	[9:0]		iRed, iGreen, iBlue;	// RGB signal from decoder
	input	[9:0]		iVGA_X, iVGA_Y;		// Current pixel location
	input				iVGA_BLANK;				// High when VGA is reading pixels
	input	[31:0]	iX, iY;					// Coordinates of pixel to detect
	input [2:0]		iColorSel;				// What color to look for
	
	parameter GREEN = 3'd0, RED = 3'd1, YELLOW = 3'd2, BLUE = 3'd3, ORANGE = 3'd4, FLASH = 3'd5;
	
	output reg	oPixel;	// High when detected
	
	wire			isColor;
	wire			isGreen, isRed, isYellow, isBlue, isOrange;
	wire			spHammer, isFlash, isSP, gHammer, rHammer, yHammer, oHammer;
	
	assign isGreen		= iGreen[9:7]>=3'b101;
	assign isRed		= iRed[9:7]>=3'b110;
	assign isYellow	= iRed[9:7]>=3'b101 && iGreen[9];
	assign isBlue		= iBlue[9:7]>=3'b100	&& iGreen[9];
	assign isOrange	= iRed[9:7]>=3'b101 && iGreen[9:7]>=3'b011;
	
	assign gHammer		= iRed[9:7]>=3'b011 && iGreen[9:7]>=3'b011 && iBlue[9:7]>=3'b010;
	assign rHammer		= iRed[9:7]>=3'b011 && iGreen[9:7]>=3'b011 && iBlue[9:7]>=3'b010;
	assign yHammer		= iRed[9:7]>=3'b011 && iGreen[9:7]>=3'b011 && iBlue[9:7]>=3'b010;
	assign oHammer		= iRed[9:7]>=3'b011 && iGreen[9:7]>=3'b011 && iBlue[9:7]>=3'b010;
	assign spHammer 	= iRed[9:7]>=3'b011 && iGreen[9:7]>=3'b011 && iBlue[9:7]>=3'b001;
	assign isFlash 	= &iRed && iGreen[9:8]==2'b11 && iBlue[9:5]==5'd0;
	assign isSP 		= iRed[9:7]>=3'b101 && iGreen[9:7]>=3'b101 && iBlue[9:7]>=3'b011;

	//4-player (all players)
	assign isColor = ((iColorSel==GREEN)  ? (isGreen	| gHammer)	: (
							(iColorSel==RED)	  ? (isRed 		| rHammer)	: (
							(iColorSel==YELLOW) ? (isYellow 	| yHammer)	: (
							(iColorSel==BLUE)	  ? (isBlue)	 				: (
							(iColorSel==ORANGE) ? (isOrange 	| oHammer)	: (
							(iColorSel==FLASH)  ? (isFlash) : (1'b0)))))));
							
	//Update detection status each frame
	always @(posedge iCLK_27) begin
		if (iVGA_BLANK) begin
			if (iVGA_Y==iY[9:0] && iVGA_X==iX[9:0]) begin
				oPixel <= isColor|isSP|spHammer;
			end
		end
	end
	
endmodule
