// Detects when a pixel turns a specified color - Specialized for vocals
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module Pixel_Detector_Vocals (
	iCLK_27,
	iRed, iGreen, iBlue,
	iVGA_X, iVGA_Y,
	iVGA_BLANK,
	iX, iY,
	iColorSel,
	oPixel,
	oLocked);
	
	input				iCLK_27;					// 27MHz clock
	input	[9:0]		iRed, iGreen, iBlue;	// RGB signal from decoder
	input	[9:0]		iVGA_X, iVGA_Y;		// Current pixel location
	input				iVGA_BLANK;				// High when VGA is reading pixels
	input	[31:0]	iX, iY;					// Coordinates of pixel to detect
	input				iColorSel;				// What color to look for
	
	parameter NOTES = 1'b1, VOICE = 1'b0;
	
	output reg	oPixel;	// High when detected
	output 		oLocked;	// High when arrow detection is locked on note
	
	reg	[7:0]	Locked;
	
	wire			isColor, isNotes, isVoice, isRed, isLocked, lockIn;
	
	assign isNotes = ( iGreen[9:7]>=3'b101 || (iRed[9:8]==2'b11 && iGreen[9:7]>=3'b100) ) && ~iBlue[9];
	assign isVoice = iGreen[9:6]>=4'b1011 && iBlue[9:6]>=4'b0111 && iRed[9:6]>=4'b1000;
	assign lockIn  = iGreen>=10'b1111111000;
	
	assign oLocked = |Locked;
	
	// Color thresholds selected by input
	assign isColor = 	iColorSel==NOTES ? isNotes :
							iColorSel==VOICE ? isVoice : 1'b0;
							
	assign isLocked = 	iColorSel==NOTES ? 1'b0 :
								iColorSel==VOICE ? (isVoice & lockIn) : 1'b0;
	
	//Update detection status each frame
	always @(posedge iCLK_27) begin
		if (iVGA_BLANK) begin
			if (iVGA_Y==iY[9:0] && iVGA_X==iX[9:0]) begin
				oPixel <= isColor;
				Locked[7:0] <= {Locked[6:0],isLocked};
			end
		end
	end
	
endmodule