////////////////////////////////////////////////////////////////////
//Rock Band Player Top Level Module											//
//Modified from Terasic's example TV decoder								//
//																						//
//Changes made by Jeff Yates (jay29) and Tom Gowing (teg25)			//
////////////////////////////////////////////////////////////////////

module RBPlayer
	(
		////////////////////	Clock Input	 	////////////////////	 
		OSC_27,							//	27 MHz
		OSC_50,							//	50 MHz
		EXT_CLOCK,						//	External Clock
		////////////////////	Push Button		////////////////////
		KEY,							//	Button[3:0]
		////////////////////	DPDT Switch		////////////////////
		DPDT_SW,						//	DPDT Switch[17:0]
		////////////////////	7-SEG Dispaly	////////////////////
		HEX0,							//	Seven Segment Digital 0
		HEX1,							//	Seven Segment Digital 1
		HEX2,							//	Seven Segment Digital 2
		HEX3,							//	Seven Segment Digital 3
		HEX4,							//	Seven Segment Digital 4
		HEX5,							//	Seven Segment Digital 5
		HEX6,							//	Seven Segment Digital 6
		HEX7,							//	Seven Segment Digital 7
		////////////////////////	LED		////////////////////////
		LED_GREEN,						//	LED Green[8:0]
		LED_RED,						//	LED Red[17:0]
		////////////////////////	UART	////////////////////////
		UART_TXD,						//	UART Transmitter
		UART_RXD,						//	UART Receiver
		////////////////////////	IRDA	////////////////////////
		IRDA_TXD,						//	IRDA Transmitter
		IRDA_RXD,						//	IRDA Receiver
		/////////////////////	SDRAM Interface		////////////////
		DRAM_DQ,						//	SDRAM Data bus 16 Bits
		DRAM_ADDR,						//	SDRAM Address bus 12 Bits
		DRAM_LDQM,						//	SDRAM Low-byte Data Mask 
		DRAM_UDQM,						//	SDRAM High-byte Data Mask
		DRAM_WE_N,						//	SDRAM Write Enable
		DRAM_CAS_N,						//	SDRAM Column Address Strobe
		DRAM_RAS_N,						//	SDRAM Row Address Strobe
		DRAM_CS_N,						//	SDRAM Chip Select
		DRAM_BA_0,						//	SDRAM Bank Address 0
		DRAM_BA_1,						//	SDRAM Bank Address 0
		DRAM_CLK,						//	SDRAM Clock
		DRAM_CKE,						//	SDRAM Clock Enable
		////////////////////	Flash Interface		////////////////
		FL_DQ,							//	FLASH Data bus 8 Bits
		FL_ADDR,						//	FLASH Address bus 22 Bits
		FL_WE_N,						//	FLASH Write Enable
		FL_RST_N,						//	FLASH Reset
		FL_OE_N,						//	FLASH Output Enable
		FL_CE_N,						//	FLASH Chip Enable
		////////////////////	SRAM Interface		////////////////
		SRAM_DQ,						//	SRAM Data bus 16 Bits
		SRAM_ADDR,						//	SRAM Adress bus 18 Bits
		SRAM_UB_N,						//	SRAM High-byte Data Mask 
		SRAM_LB_N,						//	SRAM Low-byte Data Mask 
		SRAM_WE_N,						//	SRAM Write Enable
		SRAM_CE_N,						//	SRAM Chip Enable
		SRAM_OE_N,						//	SRAM Output Enable
		////////////////////	ISP1362 Interface	////////////////
		OTG_DATA,						//	ISP1362 Data bus 16 Bits
		OTG_ADDR,						//	ISP1362 Address 2 Bits
		OTG_CS_N,						//	ISP1362 Chip Select
		OTG_RD_N,						//	ISP1362 Write
		OTG_WR_N,						//	ISP1362 Read
		OTG_RST_N,						//	ISP1362 Reset
		OTG_FSPEED,						//	USB Full Speed,	0 = Enable, Z = Disable
		OTG_LSPEED,						//	USB Low Speed, 	0 = Enable, Z = Disable
		OTG_INT0,						//	ISP1362 Interrupt 0
		OTG_INT1,						//	ISP1362 Interrupt 1
		OTG_DREQ0,						//	ISP1362 DMA Request 0
		OTG_DREQ1,						//	ISP1362 DMA Request 1
		OTG_DACK0_N,					//	ISP1362 DMA Acknowledge 0
		OTG_DACK1_N,					//	ISP1362 DMA Acknowledge 1
		////////////////////	LCD Module 16X2		////////////////
		LCD_ON,							//	LCD Power ON/OFF
		LCD_BLON,						//	LCD Back Light ON/OFF
		LCD_RW,							//	LCD Read/Write Select, 0 = Write, 1 = Read
		LCD_EN,							//	LCD Enable
		LCD_RS,							//	LCD Command/Data Select, 0 = Command, 1 = Data
		LCD_DATA,						//	LCD Data bus 8 bits
		////////////////////	SD_Card Interface	////////////////
		SD_DAT,							//	SD Card Data
		SD_DAT3,						//	SD Card Data 3
		SD_CMD,							//	SD Card Command Signal
		SD_CLK,							//	SD Card Clock
		////////////////////	USB JTAG link	////////////////////
		TDI,  							// CPLD -> FPGA (data in)
		TCK,  							// CPLD -> FPGA (clk)
		TCS,  							// CPLD -> FPGA (CS)
	    TDO,  							// FPGA -> CPLD (data out)
		////////////////////	I2C		////////////////////////////
		I2C_SDAT,						//	I2C Data
		I2C_SCLK,						//	I2C Clock
		////////////////////	PS2		////////////////////////////
		PS2_DAT,						//	PS2 Data
		PS2_CLK,						//	PS2 Clock
		////////////////////	VGA		////////////////////////////
		VGA_CLK,   						//	VGA Clock
		VGA_HS,							//	VGA H_SYNC
		VGA_VS,							//	VGA V_SYNC
		VGA_BLANK,						//	VGA BLANK
		VGA_SYNC,						//	VGA SYNC
		VGA_R,   						//	VGA Red[9:0]
		VGA_G,	 						//	VGA Green[9:0]
		VGA_B,  						//	VGA Blue[9:0]
		////////////	Ethernet Interface	////////////////////////
		ENET_DATA,						//	DM9000A DATA bus 16Bits
		ENET_CMD,						//	DM9000A Command/Data Select, 0 = Command, 1 = Data
		ENET_CS_N,						//	DM9000A Chip Select
		ENET_WR_N,						//	DM9000A Write
		ENET_RD_N,						//	DM9000A Read
		ENET_RST_N,						//	DM9000A Reset
		ENET_INT,						//	DM9000A Interrupt
		ENET_CLK,						//	DM9000A Clock 25 MHz
		////////////////	Audio CODEC		////////////////////////
		AUD_ADCLRCK,					//	Audio CODEC ADC LR Clock
		AUD_ADCDAT,						//	Audio CODEC ADC Data
		AUD_DACLRCK,					//	Audio CODEC DAC LR Clock
		AUD_DACDAT,						//	Audio CODEC DAC Data
		AUD_BCLK,						//	Audio CODEC Bit-Stream Clock
		AUD_XCK,						//	Audio CODEC Chip Clock
		////////////////	TV Decoder		////////////////////////
		TD_DATA,    					//	TV Decoder Data bus 8 bits
		TD_HS,							//	TV Decoder H_SYNC
		TD_VS,							//	TV Decoder V_SYNC
		TD_RESET,						//	TV Decoder Reset
		TD_CLK,							//	TV Decoder Line Locked Clock
		////////////////////	GPIO	////////////////////////////
		GPIO_0,							//	GPIO Connection 0
		GPIO_1							//	GPIO Connection 1
	);

////////////////////////	Clock Input	 	////////////////////////
input			OSC_27;					//	27 MHz
input			OSC_50;					//	50 MHz
input			EXT_CLOCK;				//	External Clock
////////////////////////	Push Button		////////////////////////
input	[3:0]	KEY;					//	Button[3:0]
////////////////////////	DPDT Switch		////////////////////////
input	[17:0]	DPDT_SW;				//	DPDT Switch[17:0]
////////////////////////	7-SEG Dispaly	////////////////////////
output	[6:0]	HEX0;					//	Seven Segment Digital 0
output	[6:0]	HEX1;					//	Seven Segment Digital 1
output	[6:0]	HEX2;					//	Seven Segment Digital 2
output	[6:0]	HEX3;					//	Seven Segment Digital 3
output	[6:0]	HEX4;					//	Seven Segment Digital 4
output	[6:0]	HEX5;					//	Seven Segment Digital 5
output	[6:0]	HEX6;					//	Seven Segment Digital 6
output	[6:0]	HEX7;					//	Seven Segment Digital 7
////////////////////////////	LED		////////////////////////////
output	[8:0]	LED_GREEN;				//	LED Green[8:0]
output	[17:0]	LED_RED;				//	LED Red[17:0]
////////////////////////////	UART	////////////////////////////
output			UART_TXD;				//	UART Transmitter
input			UART_RXD;				//	UART Receiver
////////////////////////////	IRDA	////////////////////////////
output			IRDA_TXD;				//	IRDA Transmitter
input			IRDA_RXD;				//	IRDA Receiver
///////////////////////		SDRAM Interface	////////////////////////
inout	[15:0]	DRAM_DQ;				//	SDRAM Data bus 16 Bits
output	[11:0]	DRAM_ADDR;				//	SDRAM Address bus 12 Bits
output			DRAM_LDQM;				//	SDRAM Low-byte Data Mask 
output			DRAM_UDQM;				//	SDRAM High-byte Data Mask
output			DRAM_WE_N;				//	SDRAM Write Enable
output			DRAM_CAS_N;				//	SDRAM Column Address Strobe
output			DRAM_RAS_N;				//	SDRAM Row Address Strobe
output			DRAM_CS_N;				//	SDRAM Chip Select
output			DRAM_BA_0;				//	SDRAM Bank Address 0
output			DRAM_BA_1;				//	SDRAM Bank Address 0
output			DRAM_CLK;				//	SDRAM Clock
output			DRAM_CKE;				//	SDRAM Clock Enable
////////////////////////	Flash Interface	////////////////////////
inout	[7:0]	FL_DQ;					//	FLASH Data bus 8 Bits
output	[21:0]	FL_ADDR;				//	FLASH Address bus 22 Bits
output			FL_WE_N;				//	FLASH Write Enable
output			FL_RST_N;				//	FLASH Reset
output			FL_OE_N;				//	FLASH Output Enable
output			FL_CE_N;				//	FLASH Chip Enable
////////////////////////	SRAM Interface	////////////////////////
inout	[15:0]	SRAM_DQ;				//	SRAM Data bus 16 Bits
output	[17:0]	SRAM_ADDR;				//	SRAM Address bus 18 Bits
output			SRAM_UB_N;				//	SRAM High-byte Data Mask 
output			SRAM_LB_N;				//	SRAM Low-byte Data Mask 
output			SRAM_WE_N;				//	SRAM Write Enable
output			SRAM_CE_N;				//	SRAM Chip Enable
output			SRAM_OE_N;				//	SRAM Output Enable
////////////////////	ISP1362 Interface	////////////////////////
inout	[15:0]	OTG_DATA;				//	ISP1362 Data bus 16 Bits
output	[1:0]	OTG_ADDR;				//	ISP1362 Address 2 Bits
output			OTG_CS_N;				//	ISP1362 Chip Select
output			OTG_RD_N;				//	ISP1362 Write
output			OTG_WR_N;				//	ISP1362 Read
output			OTG_RST_N;				//	ISP1362 Reset
output			OTG_FSPEED;				//	USB Full Speed,	0 = Enable, Z = Disable
output			OTG_LSPEED;				//	USB Low Speed, 	0 = Enable, Z = Disable
input			OTG_INT0;				//	ISP1362 Interrupt 0
input			OTG_INT1;				//	ISP1362 Interrupt 1
input			OTG_DREQ0;				//	ISP1362 DMA Request 0
input			OTG_DREQ1;				//	ISP1362 DMA Request 1
output			OTG_DACK0_N;			//	ISP1362 DMA Acknowledge 0
output			OTG_DACK1_N;			//	ISP1362 DMA Acknowledge 1
////////////////////	LCD Module 16X2	////////////////////////////
inout	[7:0]	LCD_DATA;				//	LCD Data bus 8 bits
output			LCD_ON;					//	LCD Power ON/OFF
output			LCD_BLON;				//	LCD Back Light ON/OFF
output			LCD_RW;					//	LCD Read/Write Select, 0 = Write, 1 = Read
output			LCD_EN;					//	LCD Enable
output			LCD_RS;					//	LCD Command/Data Select, 0 = Command, 1 = Data
////////////////////	SD Card Interface	////////////////////////
inout			SD_DAT;					//	SD Card Data
inout			SD_DAT3;				//	SD Card Data 3
inout			SD_CMD;					//	SD Card Command Signal
output			SD_CLK;					//	SD Card Clock
////////////////////////	I2C		////////////////////////////////
inout			I2C_SDAT;				//	I2C Data
output			I2C_SCLK;				//	I2C Clock
////////////////////////	PS2		////////////////////////////////
input		 	PS2_DAT;				//	PS2 Data
input			PS2_CLK;				//	PS2 Clock
////////////////////	USB JTAG link	////////////////////////////
input  			TDI;					// CPLD -> FPGA (data in)
input  			TCK;					// CPLD -> FPGA (clk)
input  			TCS;					// CPLD -> FPGA (CS)
output 			TDO;					// FPGA -> CPLD (data out)
////////////////////////	VGA			////////////////////////////
output			VGA_CLK;   				//	VGA Clock
output			VGA_HS;					//	VGA H_SYNC
output			VGA_VS;					//	VGA V_SYNC
output			VGA_BLANK;				//	VGA BLANK
output			VGA_SYNC;				//	VGA SYNC
output	[9:0]	VGA_R;   				//	VGA Red[9:0]
output	[9:0]	VGA_G;	 				//	VGA Green[9:0]
output	[9:0]	VGA_B;   				//	VGA Blue[9:0]
////////////////	Ethernet Interface	////////////////////////////
inout	[15:0]	ENET_DATA;				//	DM9000A DATA bus 16Bits
output			ENET_CMD;				//	DM9000A Command/Data Select, 0 = Command, 1 = Data
output			ENET_CS_N;				//	DM9000A Chip Select
output			ENET_WR_N;				//	DM9000A Write
output			ENET_RD_N;				//	DM9000A Read
output			ENET_RST_N;				//	DM9000A Reset
input			ENET_INT;				//	DM9000A Interrupt
output			ENET_CLK;				//	DM9000A Clock 25 MHz
////////////////////	Audio CODEC		////////////////////////////
inout			AUD_ADCLRCK;			//	Audio CODEC ADC LR Clock
input			AUD_ADCDAT;				//	Audio CODEC ADC Data
inout			AUD_DACLRCK;			//	Audio CODEC DAC LR Clock
output			AUD_DACDAT;				//	Audio CODEC DAC Data
inout			AUD_BCLK;				//	Audio CODEC Bit-Stream Clock
output			AUD_XCK;				//	Audio CODEC Chip Clock
////////////////////	TV Devoder		////////////////////////////
input	[7:0]	TD_DATA;    			//	TV Decoder Data bus 8 bits
input			TD_HS;					//	TV Decoder H_SYNC
input			TD_VS;					//	TV Decoder V_SYNC
output			TD_RESET;				//	TV Decoder Reset
input			TD_CLK;					//	TV Decoder Line Locked Clock
////////////////////////	GPIO	////////////////////////////////
inout	[35:0]	GPIO_0;					//	GPIO Connection 0
inout	[35:0]	GPIO_1;					//	GPIO Connection 1
////////////////////////////////////////////////////////////////////

//	Enable TV Decoder
assign	TD_RESET	=	KEY[0];

//	For Audio CODEC
wire		AUD_CTRL_CLK;	//	For Audio Controller
assign		AUD_XCK	=	AUD_CTRL_CLK;

//	Audio CODEC and video decoder setting
I2C_AV_Config 		u1	(	//	Host Side
							.iCLK(OSC_50),
							.iRST_N(KEY[0]),
							//	I2C Side
							.I2C_SCLK(I2C_SCLK),
							.I2C_SDAT(I2C_SDAT)	);

//	TV Decoder Stable Check
TD_Detect			u2	(	.oTD_Stable(TD_Stable),
							.iTD_VS(TD_VS),
							.iTD_HS(TD_HS),
							.iRST_N(KEY[0])	);

//	Reset Delay Timer
Reset_Delay			u3	(	.iCLK(OSC_50),
							.iRST(TD_Stable),
							.oRST_0(DLY0),
							.oRST_1(DLY1),
							.oRST_2(DLY2));

//	ITU-R 656 to YUV 4:2:2
ITU_656_Decoder		u4	(	//	TV Decoder Input
							.iTD_DATA(TD_DATA),
							//	Position Output
							.oTV_X(TV_X),
							//	YUV 4:2:2 Output
							.oYCbCr(YCbCr),
							.oDVAL(TV_DVAL),
							//	Control Signals
							.iSwap_CbCr(Quotient[0]),
							.iSkip(Remain==4'h0),
							.iRST_N(DLY1),
							.iCLK_27(TD_CLK)	);

//	For Down Sample 720 to 640
DIV 				u5	(	.aclr(!DLY0),	
							.clock(TD_CLK),
							.denom(4'h9),
							.numer(TV_X),
							.quotient(Quotient),
							.remain(Remain));

//	SDRAM frame buffer
wire fake_wr;
assign fake_wr = DPDT_SW[10] ? 1'b0 : TV_DVAL;
//assign fake_wr = (dir & DPDT_SW[10]) ? 1'b0 : TV_DVAL;

Sdram_Control_4Port	u6	(	//	HOST Side
						    .REF_CLK(OSC_27),
							.CLK_18(AUD_CTRL_CLK),
						    .RESET_N(1'b1),
							//	FIFO Write Side 1
						    .WR1_DATA(YCbCr),
							.WR1(fake_wr),
							.WR1_FULL(WR1_FULL),
							.WR1_ADDR(0),
							.WR1_MAX_ADDR(640*507),		//	525-18
							.WR1_LENGTH(9'h80),
							.WR1_LOAD(!DLY0),
							.WR1_CLK(TD_CLK),
							//	FIFO Read Side 1
						    .RD1_DATA(m1YCbCr),
				        	.RD1(m1VGA_Read),
				        	.RD1_ADDR(640*13),			//	Read odd field and bypess blanking
							.RD1_MAX_ADDR(640*253),
							.RD1_LENGTH(9'h80),
				        	.RD1_LOAD(!DLY0),
							.RD1_CLK(OSC_27),
							//	FIFO Read Side 2
						    .RD2_DATA(m2YCbCr),
				        	.RD2(m2VGA_Read),
				        	.RD2_ADDR(640*267),			//	Read even field and bypess blanking
							.RD2_MAX_ADDR(640*507),
							.RD2_LENGTH(9'h80),
				        	.RD2_LOAD(!DLY0),
							.RD2_CLK(OSC_27),
							//	SDRAM Side
						    .SA(DRAM_ADDR),
						    .BA({DRAM_BA_1,DRAM_BA_0}),
						    .CS_N(DRAM_CS_N),
						    .CKE(DRAM_CKE),
						    .RAS_N(DRAM_RAS_N),
				            .CAS_N(DRAM_CAS_N),
				            .WE_N(DRAM_WE_N),
						    .DQ(DRAM_DQ),
				            .DQM({DRAM_UDQM,DRAM_LDQM}),
							.SDR_CLK(DRAM_CLK)	);

//	YUV 4:2:2 to YUV 4:4:4
YUV422_to_444		u7	(	//	YUV 4:2:2 Input
							.iYCbCr(mYCbCr),
							//	YUV	4:4:4 Output
							.oY(mY),
							.oCb(mCb),
							.oCr(mCr),
							//	Control Signals
							.iX(VGA_X),
							.iCLK(OSC_27),
							.iRST_N(DLY0));

//	YCbCr 8-bit to RGB-10 bit 
YCbCr2RGB 			u8	(	//	Output Side
							.Red(mRed),
							.Green(mGreen),
							.Blue(mBlue),
							.oDVAL(mDVAL),
							//	Input Side
							.iY(mY),
							.iCb(mCb),
							.iCr(mCr),
							.iDVAL(VGA_Read),
							//	Control Signal
							.iRESET(!DLY2),
							.iCLK(OSC_27));
							
//Cursor Module/////////////////////////////////////
wire [9:0]	cRed, cGreen, cBlue;	//RGB output from Cursor module
wire [9:0]	CursorX, CursorY;		//Location of Cursor
wire [9:0]  Color;

Cursor c0 (
	.iCLK					(OSC_27),
	.iReset				(~KEY[0]),
	.iVGA_X				(VGA_X),		//X,Y from VGA_Ctrl
	.iVGA_Y				(VGA_Y),
	.iRed					(mRed),		//RGB from SRAM
	.iGreen				(mGreen),
	.iBlue				(mBlue),
	.iRGBsel				(DPDT_SW[16:14]),
	.iMoveCursor		(~KEY[3]),
	.iCursorDir			(DPDT_SW[4:0]),
	.iGxy_G				(Gxy_G),	
	.iRxy_G				(Rxy_G),	
	.iYxy_G				(Yxy_G),	
	.iBxy_G				(Bxy_G),	
	.iOxy_G				(Oxy_G),	
	.iGxy_B				(Gxy_B),	
	.iRxy_B				(Rxy_B),	
	.iYxy_B				(Yxy_B),	
	.iBxy_B				(Bxy_B),	
	.iOxy_B				(Oxy_B),
	.iGxy_D				(Gxy_D),	
	.iRxy_D				(Rxy_D),	
	.iYxy_D				(Yxy_D),	
	.iBxy_D				(Bxy_D),	
	.iPresetSwitches	(DPDT_SW[9:5]),
	.oRed					(cRed),
	.oGreen				(cGreen),
	.oBlue				(cBlue),
	.oColor				(Color),
	.oCursorX			(CursorX),
	.oCursorY			(CursorY)
);

HexDigit h7(HEX7, 4'd0);
HexDigit h6(HEX6, CursorX/100);
HexDigit h5(HEX5, (CursorX%100)/10);
HexDigit h4(HEX4, CursorX%10);
HexDigit h3(HEX3, 4'd0);
HexDigit h2(HEX2, CursorY/100);
HexDigit h1(HEX1, (CursorY%100)/10);
HexDigit h0(HEX0, CursorY%10);

//Frame Counter module******************************************************
wire 			newFrame;
wire	[7:0]	frameCount;

FrameCounter fc0 (
	.iCLK			(OSC_27),
	.iReset		(~KEY[0]),
	.iVGA_X		(VGA_X),
	.iVGA_Y		(VGA_Y),
	.iVGA_BLANK	(VGA_BLANK),
	.oNewFrame	(newFrame),
	.oFrameCount(frameCount)
);

//Wireless Module***********************************************************
Wireless w0(
	.iCLK_50			(OSC_50),
	.iReset			(~KEY[0]),
	.iNewFrame		(newFrame),
	.iGuitarButs_x	({2'd0,~{G_Strum,G_Or, G_Bl, G_Ye, G_Re, G_Gr}}),
	.iBassButs_x	({2'd0,~{B_Strum,B_Or, B_Bl, B_Ye, B_Re, B_Gr}}),
	.iDrumButs_x	({3'd0,~{D_Strum,D_Ye, D_Bl, D_Re, D_Gr}}),
	.oUART_TXD		(GPIO_0[0])
);

//Detector alignment********************************************************
wire [19:0]	Gxy_G,Rxy_G,Yxy_G,Bxy_G,Oxy_G;

//4-player mode Guitar (left-most player)
assign Gxy_G = {10'd110,10'd316};
assign Rxy_G = {10'd133,10'd315};
assign Yxy_G = {10'd155,10'd315};
assign Bxy_G = {10'd177,10'd315};
assign Oxy_G = {10'd199,10'd316};

//Guitar Module///////////////////////////////////////
wire G_Gr,G_Re,G_Ye,G_Bl,G_Or;
wire G_Strum;

Guitar g0(
	.iCLK_27			(OSC_27),
	.iReset			(~KEY[0]),
	.iNewFrame		(newFrame),
	.iFrameCount	(frameCount),
	.iRed 			(mRed), 
	.iGreen 			(mGreen), 
	.iBlue 			(mBlue),
	.iVGA_X 			(VGA_X), 
	.iVGA_Y 			(VGA_Y),
	.iVGA_BLANK 	(VGA_BLANK),
	.iGxy				(Gxy_G),
	.iRxy				(Rxy_G),
	.iYxy				(Yxy_G),
	.iBxy				(Bxy_G),
	.iOxy				(Oxy_G),
	.oNotes			({G_Gr, G_Re, G_Ye, G_Bl, G_Or}),
	.oStrum			(G_Strum)
);

// Bass Module ///////////////////////////////////////////////////////////////////////
wire [19:0]	Gxy_B,Rxy_B,Yxy_B,Bxy_B,Oxy_B;
wire B_Gr,B_Re,B_Ye,B_Bl,B_Or;
wire B_Strum;

//4-player mode Bass (right-most player)
assign Gxy_B = {10'd450,10'd316};
assign Rxy_B = {10'd473,10'd315};
assign Yxy_B = {10'd494,10'd315};
assign Bxy_B = {10'd516,10'd315};
assign Oxy_B = {10'd538,10'd316};

Guitar b0(
	.iCLK_27			(OSC_27),
	.iReset			(~KEY[0]),
	.iNewFrame		(newFrame),
	.iFrameCount	(frameCount),
	.iRed 			(mRed), 
	.iGreen 			(mGreen), 
	.iBlue 			(mBlue),
	.iVGA_X 			(VGA_X), 
	.iVGA_Y 			(VGA_Y),
	.iVGA_BLANK 	(VGA_BLANK),
	.iGxy				(Gxy_B),
	.iRxy				(Rxy_B),
	.iYxy				(Yxy_B),
	.iBxy				(Bxy_B),
	.iOxy				(Oxy_B),
	.oNotes			({B_Gr, B_Re, B_Ye, B_Bl, B_Or}),
	.oStrum			(B_Strum)
);
// Drums Module //////////////////////////////////////////////////////////////////////
wire [19:0]	Gxy_D,Rxy_D,Yxy_D,Bxy_D;
wire D_Gr,D_Re,D_Ye,D_Bl;
wire D_Strum;
wire [3:0] pedal_dummy, pedal_LED;

//4-player mode Drums (center player)
assign Rxy_D = {10'd283,10'd316};
assign Yxy_D = {10'd311,10'd315};
assign Bxy_D = {10'd338,10'd315};
assign Gxy_D = {10'd364,10'd316};

Drums d0(
	.iCLK_27		(OSC_27),
	.iReset		(~KEY[0]),
	.iNewFrame	(newFrame),
	.iFrameCount(frameCount),
	.iRed 		(mRed), 
	.iGreen 		(mGreen), 
	.iBlue 		(mBlue),
	.iVGA_X 		(VGA_X), 
	.iVGA_Y 		(VGA_Y),
	.iVGA_BLANK (VGA_BLANK),
	.iGxy			(Gxy_D),
	.iRxy			(Rxy_D),
	.iYxy			(Yxy_D),
	.iBxy			(Bxy_D),
	.oNotes		({D_Gr, D_Re, D_Ye, D_Bl}),
	.oPedals		(pedal_dummy),
	.pedal_LED	(pedal_LED),
	.oStrum		(D_Strum)
);
// Vocals Module /////////////////////////////////////////////////////////////////////
Vocals v0(
	.iCLK_27		(OSC_27),
	.iCLK_50		(OSC_50),
	.iRST			(~KEY[0]),
	.iNewFrame	(newFrame),
	.iRed			(mRed),
	.iGreen		(mGreen),
	.iBlue		(mBlue),
	.iVGA_X		(VGA_X),
	.iVGA_Y		(VGA_Y),
	.iVGA_BLANK	(VGA_BLANK),
	.iPxSel		(DPDT_SW[11]),
	.oDir			(dir),
	.oPixels		(Pixels),
	.oLevel		(Level),
	.oIsVoice	(isVoice),
	.oIsNotes	(isNotes),
	.oAudio_out	(audio_out) 
);

wire	[32:0]	Pixels;
wire				dir;
wire	[5:0]		Level;
	
assign LED_RED[9:0] = DPDT_SW[17] ? Color : {{2{1'b0}},{2{pedal_LED[3]}},{2{pedal_LED[2]}},{2{pedal_LED[1]}},{2{pedal_LED[0]}}}	;

assign LED_GREEN[7:0] = {8{D_Strum}};
assign LED_GREEN[8] = dir;

//	VGA Controller
VGA_Ctrl			u9	(	//	Host Side
							.iRed(cRed),
							.iGreen(cGreen),
							.iBlue(cBlue),
							.oCurrent_X(VGA_X),
							.oCurrent_Y(VGA_Y),
							.oRequest(VGA_Read),
							//	VGA Side
							.oVGA_R(VGA_R),
							.oVGA_G(VGA_G),
							.oVGA_B(VGA_B),
							.oVGA_HS(VGA_HS),
							.oVGA_VS(VGA_VS),
							.oVGA_SYNC(VGA_SYNC),
							.oVGA_BLANK(VGA_BLANK),
							.oVGA_CLOCK(VGA_CLK),
							//	Control Signal
							.iCLK(OSC_27),
							.iRST_N(DLY2)	);

//	For ITU-R 656 Decoder
wire	[15:0]	YCbCr;
wire	[9:0]	TV_X;
wire			TV_DVAL;

//	For VGA Controller
wire	[9:0]	mRed;
wire	[9:0]	mGreen;
wire	[9:0]	mBlue;
wire	[10:0]	VGA_X;
wire	[10:0]	VGA_Y;
wire			VGA_Read;	//	VGA data request
wire			m1VGA_Read;	//	Read odd field
wire			m2VGA_Read;	//	Read even field

//	For YUV 4:2:2 to YUV 4:4:4
wire	[7:0]	mY;
wire	[7:0]	mCb;
wire	[7:0]	mCr;

//	For field select
wire	[15:0]	mYCbCr;
wire	[15:0]	mYCbCr_d;
wire	[15:0]	m1YCbCr;
wire	[15:0]	m2YCbCr;
wire	[15:0]	m3YCbCr;

//	For Delay Timer
wire			TD_Stable;
wire			DLY0;
wire			DLY1;
wire			DLY2;

//	For Down Sample
wire	[3:0]	Remain;
wire	[9:0]	Quotient;

assign	m1VGA_Read	=	VGA_Y[0]		?	1'b0		:	VGA_Read	;
assign	m2VGA_Read	=	VGA_Y[0]		?	VGA_Read	:	1'b0		;
assign	mYCbCr_d	=	!VGA_Y[0]		?	m1YCbCr		:
											m2YCbCr		;
assign	mYCbCr		=	m5YCbCr;

wire			mDVAL;

//	Line buffer, delay one line
Line_Buffer u10	(	.clken(VGA_Read),
					.clock(OSC_27),
					.shiftin(mYCbCr_d),
					.shiftout(m3YCbCr));

Line_Buffer u11	(	.clken(VGA_Read),
					.clock(OSC_27),
					.shiftin(m3YCbCr),
					.shiftout(m4YCbCr));

wire	[15:0]	m4YCbCr;
wire	[15:0]	m5YCbCr;
wire	[8:0]	Tmp1,Tmp2;
wire	[7:0]	Tmp3,Tmp4;

assign	Tmp1	=	m4YCbCr[7:0]+mYCbCr_d[7:0];
assign	Tmp2	=	m4YCbCr[15:8]+mYCbCr_d[15:8];
assign	Tmp3	=	Tmp1[8:2]+m3YCbCr[7:1];
assign	Tmp4	=	Tmp2[8:2]+m3YCbCr[15:9];
assign	m5YCbCr	=	{Tmp4,Tmp3};
					
///////////////////////////////////////////////////////////////////////////////
// Audio DAC
//
wire signed [15:0] audio_out; //output to audio DAC
assign AUD_ADCLRCK = AUD_DACLRCK;

wire DLY_RST_AUD_N;
Reset_Delay_Audio rda0 ( OSC_50, DLY_RST_AUD_N);

AUDIO_DAC_ADC 	audio_dac1	(	//	Audio Side
	.oAUD_BCK		(AUD_BCLK),
	.oAUD_DATA		(AUD_DACDAT),
	.oAUD_LRCK		(AUD_DACLRCK),
	.oAUD_inL		( ), // audio data from ADC 
	.oAUD_inR		( ), // audio data from ADC 
	.iAUD_ADCDAT	(AUD_ADCDAT),
	.iAUD_extL		(audio_out), // audio data to DAC
	.iAUD_extR		(audio_out), // audio data to DAC
	//	Control Signals
	.iCLK_18_4		(AUD_CTRL_CLK),
	.iRST_N			(DLY_RST_AUD_N) 
);

endmodule
