//********************************************************** 
// Receiver code to receive data from RS232-like radio
// Jeff Yates and Tom Gowing
// Cornell University 2011

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

volatile char rxByte;		// Received Byte
volatile char rxDeviceId;	// ID of device to receive data

//********************************************************** 
// Receive Complete ISR
ISR (USART0_RX_vect) {
	rxByte = UDR0;
	if (rxByte>=0x40) {
		rxDeviceId = rxByte;
	} 
	else if (rxDeviceId==0x40) {
		PORTC=~rxByte;
	}
}

//********************************************************** 
// Entry Point and loop
int main(void) {

	DDRC = 0xff;	// Port C is output to instrument
	DDRB = 0x00;	// Port B is input with instrument ID
	DDRD = 0x04;	// Port D is input. D.0 is RX0 input pin. D.2 is LED

	
	UCSR0B = (1<<RXCIE0) | (1<<RXEN0);	// RX Complete Interrupt Enable and Receiver Enable.
	UCSR0C = (1<<UCSZ01) | (1<<UCSZ00);	// Set to 8 bit character length
	
	// Set Baud rate prescalar to 103 w/ 16MHz clk = 9600 Baud
	UBRR0H = 0b00000000;	
	UBRR0L = 0b01100111;
	
	rxDeviceId = 0;	// Init Device ID
	PORTC = 0;	// Release all instrument buttons
	
	sei();	// Enable the interrupts

	PORTD = 0;
	
	while(1){
		// Do nothing
	}
}
