// FIFO buffer to store notes to be strummed
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module Ring_Buffer (
	iCLK_27,
	iReset,
	iNotes,
	iFrameNum,
	iWriteBuffer,
	iReadBuffer,
	oNotes,
	oFrameNum,
	oValid	);
	
	input 		iCLK_27;
	input 		iReset;
	input	[4:0]	iNotes;			// Notes to strum {g,r,y,b,o}
	input	[7:0]	iFrameNum;		// Frame number to strum on
	input 		iWriteBuffer;	// Signal to write
	input 		iReadBuffer;	// Signal to read
	
	output 	[4:0]	oNotes;
	output	[7:0]	oFrameNum;
	output 			oValid;		// The buffer is not empty
	
	reg	[3:0]	head, tail;		// Ring buffer pointers
	
	reg	[4:0]	notes		[15:0];	//	Memory for notes
	reg	[7:0]	frameNum	[15:0];	// Memory for frame numbers
	
	parameter MERGE_DELAY = 8'd2;	//Min delay between notes to be considered separate notes
	
	assign oNotes = notes[head];
	assign oFrameNum = frameNum[head];
	assign oValid = (head==tail) ? 1'b0 : 1'b1;	//Buffer is empty when head = tail
	
	always @(posedge iCLK_27) begin
		if (iReset) begin
			head <= 4'b0;
			tail <= 4'b0;	
		end
		else begin
			if (iWriteBuffer) begin
				// If notes are too close in time, merge into 1 strum entry
				if (iFrameNum-frameNum[tail-4'd1] <= MERGE_DELAY) begin
					notes[tail-4'd1] <= notes[tail-4'd1] | iNotes;
					tail <= tail;
				end
				// Otherwise make new entry and increment tail
				else begin
					notes[tail] <= iNotes;
					frameNum[tail] <= iFrameNum;
					tail <= tail + 4'b1;
				end
			end
			if (iReadBuffer) begin
				head <= head + 4'b1;	// Increment head on read			
			end
		end
	end
	
endmodule
	
