// A vertical strip of pixel detectors to determine location of notes/voice
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module Strip_Detector (
	iCLK_27,
	iRed, iGreen, iBlue,
	iVGA_X, iVGA_Y,
	iVGA_BLANK,
	iBotLeft,
	iColorSel,
	oPixels,
	oLocked);
	
	input				iCLK_27;					//	27MHz clock
	input	[9:0]		iRed, iGreen, iBlue;	//	RGB signal from decoder
	input	[9:0]		iVGA_X, iVGA_Y;		//	Current pixel location
	input				iVGA_BLANK;				//	High when VGA is reading pixels
	input	[19:0]	iBotLeft;				// Bottom-Left of pixel grid box {x,y}
	input 			iColorSel;				// Selects which color to detect
	
	output [31:0] 	oPixels;	// Pixel detection output array
	output [31:0]	oLocked; // Array of locked in bits
	
	`define STRIP_HEIGHT 62	// Height of strip in pixels

	wire	[9:0]	BL_X, BL_Y;
	
	assign BL_X = iBotLeft[19:10];
	assign BL_Y = iBotLeft[9:0];

	//Generate pixel detector strip
	genvar y;
	generate
		for(y = 0; y <= `STRIP_HEIGHT; y = y + 2) begin:yFor
			Pixel_Detector_Vocals pdv(
				.iCLK_27		(iCLK_27),
				.iRed			(iRed),
				.iGreen		(iGreen),
				.iBlue		(iBlue),
				.iVGA_X		(iVGA_X),
				.iVGA_Y		(iVGA_Y),
				.iVGA_BLANK	(iVGA_BLANK),
				.iX			({22'b0,BL_X}),
				.iY			({22'b0,BL_Y}-y),
				.iColorSel	(iColorSel),
				.oPixel		(oPixels[y/2]),
				.oLocked		(oLocked[y/2])
			);
		end
	endgenerate

endmodule
