// Strums on frame match from buffer output
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module Strummer (
	iCLK,
	iNewFrame,
	iFrameCount,
	iBufferFrameCount,
	iValid,
	oRead,
	oStrum );
	
	input 		iCLK;
	input			iNewFrame;				// Signals a new frame
	input	[7:0]	iFrameCount;			//	Global frame counter
	input	[7:0]	iBufferFrameCount;	//	Frame value of next note to strum
	input			iValid;					// Is the head value of buffer valid?

	output reg oRead;		// Tells buffer to increment head after a read
	output reg oStrum;	// Strum output to guitar
	
	reg Hold;	// History used to hold strum for 2 frames
	
	// On a frame count match with buffer, hold strummer high for 2 frames
	always @(posedge iCLK) begin
		if (iNewFrame && ~Hold) begin
			oStrum <= ((iFrameCount==(iBufferFrameCount))&&iValid) ? 1'b1 : 1'b0;
			oRead  <= ((iFrameCount==(iBufferFrameCount))&&iValid) ? 1'b1 : 1'b0;
			Hold	 <= ((iFrameCount==(iBufferFrameCount))&&iValid) ? 1'b1 : 1'b0;
		end
		else if (iNewFrame && Hold) begin
			oStrum <= 1'b1;
			oRead  <= 1'b0;
			Hold	 <= 1'b0;
		end
		else begin
			oStrum <= oStrum;
			oRead  <= 1'b0;
		end
	end
	
endmodule
