module Vocals (
	iCLK_27, iCLK_50,
	iRST,
	iNewFrame,
	iRed,iGreen,iBlue,
	iVGA_X,iVGA_Y,
	iVGA_BLANK,
	iPxSel,
	oDir,
	oPixels,
	oLevel,
	oIsVoice, oIsNotes,
	oAudio_out );
	
	input			iCLK_27, iCLK_50;
	input			iRST;
	input			iNewFrame;
	input	[9:0]	iRed, iGreen, iBlue;
	input	[9:0]	iVGA_X, iVGA_Y;
	input			iVGA_BLANK;
	input			iPxSel;
		
	output [15:0]	oAudio_out;
	output			oIsVoice, oIsNotes;
	
	parameter NOTES=1'b1, VOICE=1'b0;
	
	
	output	[31:0]	oPixels;
	output				oDir;
	output	[5:0]		oLevel;
	assign oPixels = iPxSel ? Pixels_Notes : Pixels_Voice;
	assign oDir = Lock_in;
	
	// Strip Detectors /////////////////////////////////////////////////////////
	wire 	[31:0]	Pixels_Voice, Pixels_Notes;
	wire 	[31:0]	Locked_Voice, Locked_Notes;
	wire				Lock_in;
	wire	[19:0]	xy_voice, xy_notes;	//Bottom-left coordinate for detector strip {x,y}
	
	assign xy_voice = {10'd100 , 10'd90};
	assign xy_notes = {10'd150 , 10'd90};
	
	assign oIsVoice = |Pixels_Voice;
	assign oIsNotes = |Pixels_Notes;
	assign Lock_in = |Locked_Voice;
	
	Strip_Detector sd_voice (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(xy_voice),
		.iColorSel	(VOICE),
		.oPixels		(Pixels_Voice),
		.oLocked		(Locked_Voice)
	);
	
	Strip_Detector sd_notes (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(xy_notes),
		.iColorSel	(NOTES),
		.oPixels		(Pixels_Notes),
		.oLocked		(Locked_Notes)
	);
	
	// Detector Comparison /////////////////////////////////////////////////////
	reg	[31:0]	px_v_shifted, px_n_shifted;
	reg	[31:0]	px_v_final, px_n_final;
	reg				dir;	// Direction to change frequency. Up=1, Down=0
	
	// Shift down pixel arrays to scale magnitude
	always @(posedge iCLK_27) begin
		if (iNewFrame) begin
			px_v_shifted <= Pixels_Voice;
			px_n_shifted <= Pixels_Notes;
		end
		else if (px_v_shifted[0]==1'b0 && px_n_shifted[0]==1'b0) begin
			px_v_shifted <= px_v_shifted >> 1;
			px_n_shifted <= px_n_shifted >> 1;
		end
		else begin
			dir <= px_v_shifted[0];
		end
	end
	
	// Increment Adjustment LUT ////////////////////////////////////////////////
	Note_LUT nlut0(
		.iCLK_27		(iCLK_27),
		.iPx_v		(px_v_shifted), 
		.iPx_n		(px_n_shifted),
		.iDir			(dir),
		.iNewFrame	(iNewFrame),
		.iLockIn		(Lock_in),
		.oIncr		(DDS_inc),
		.oLevel		(oLevel)
	);
	
	// DDS Logic ///////////////////////////////////////////////////////////////
	reg	[31:0]	DDS_acc;	// 32 bit accumulator register
	wire	[31:0]	DDS_inc;	// 32 bit increment
	wire	[31:0]	DDS_adj;	// 32 bit increment adjustment value

	always @(posedge iCLK_50) begin
		DDS_acc <= DDS_acc + DDS_inc;
	end
	
	// Sine ROM
	sync_rom sr0 (iCLK_50, DDS_acc[31:24], oAudio_out);
	
	
endmodule
